<?php

class ModuleNotify extends AppModule
{
  function __construct($System)
  {
    parent::__construct($System);
    $this->Name = 'Notify';
    $this->Version = '1.0';
    $this->Creator = 'Chronos';
    $this->License = 'GNU/GPL';
    $this->Description = 'Send notification messages to selected users';
    $this->Dependencies = array('User', 'Subject');
  }

  function DoStart()
  {
    $this->System->FormManager->RegisterClass('NotifyUser', array(
      'Title' => 'Upozornění uživatelé',
      'Table' => 'NotifyUser',
      'Items' => array(
        'User' => array('Type' => 'TUser', 'Caption' => 'Uživatel', 'Default' => ''),
        'Contact' => array('Type' => 'TContact', 'Caption' => 'Kontakt', 'Default' => ''),
        'Period' => array('Type' => 'Integer', 'Caption' => 'Interval', 'Default' => '60'),
      ),
    ));
    $this->System->FormManager->RegisterClass('NotifyCategory', array(
      'Title' => 'Kategorie upozornění',
      'Table' => 'NotifyCategory',
      'Items' => array(
        'Name' => array('Type' => 'String', 'Caption' => 'Jméno', 'Default' => ''),
        'SysName' => array('Type' => 'String', 'Caption' => 'Systémové jméno', 'Default' => ''),
      ),
    ));
  }

  function Check()
  {
    $OnlineText = array('<span style="color:red;">Nedostupný</span>', '<span style="color:green;">Dostupný</span>');
    $Output = '';
    $DbResult = $this->Database->query('SELECT `NotifyUser`.`Id`, `NotifyUser`.`LastTime`, `User`.`Name`, `Contact`.`Value`, `Contact`.`Category` FROM `NotifyUser` '.
      'LEFT JOIN `User` ON `User`.`Id` = `NotifyUser`.`User` '.
      'LEFT JOIN `Contact` ON `Contact`.`Id` = `NotifyUser`.`Contact`');
    while($User = $DbResult->fetch_assoc())
    {
      $Time = time();
      $Content = '';
      $Output .= 'User '.$User['Name'].'<br/>';
      $DbResult2 = $this->Database->query('SELECT `Id`, `Name`, `SysName` FROM `NotifyCategory`');
      while($Category = $DbResult2->fetch_assoc())
      {
        if($Category['SysName'] == 'NetworkReachability') {
          $DbResult3 = $this->Database->query('SELECT CONCAT_WS("-", `NetworkDevice`.`Name`, NULLIF(`NetworkInterface`.`Name`, "")) AS `Name`, '.
            '`NetworkInterfaceUpDown`.`State`, `NetworkInterfaceUpDown`.`Duration`, `NetworkInterfaceUpDown`.`Time` FROM `NetworkInterfaceUpDown` '.
            'LEFT JOIN `NetworkInterface` ON `NetworkInterface`.`Id` = `NetworkInterfaceUpDown`.`Interface` '.
            'LEFT JOIN `NetworkDevice` ON `NetworkDevice`.`Id` = `NetworkInterface`.`Device` '.
            'WHERE (`Time` > "'.$User['LastTime'].'") AND (`Time` <= "'.TimeToMysqlDateTime($Time).'") '.
            'AND (`NetworkDevice`.`PermanentOnline`=1) ORDER BY `NetworkInterfaceUpDown`.`Time` DESC');
          if($DbResult3->num_rows > 0) {
            $Output .= 'Category '.$Category['Name'].":\n";
            $Content .= 'Category '.$Category['Name']."<br>\n";
            $Content .= '<table><tr><th align="center">Jméno</th><th align="center">Stav</th><th align="center">Čas</th><th align="center">Trvání</th></tr>'."\n";
            while($Item = $DbResult3->fetch_assoc())
            {
              $Duration = sprintf('%02d', floor($Item['Duration'] / 3600 % 24)).':'.
                sprintf('%02d', floor($Item['Duration'] / 60 % 60)).':'.
                sprintf('%02d', floor($Item['Duration'] % 60));
              $Days = floor($Item['Duration'] / (60 * 60 * 24));
              if($Days > 0) $Duration = $Days.' dnů '.$Duration;

              $Content .= '<tr><td>'.$Item['Name'].'</td><td>'.$OnlineText[$Item['State']].
                '</td><td>'.$Item['Time'].'</td><td>'.$Duration.'</td></tr>'."\n";
            }
            $Content .= '</table>'."\n";
          }
        }
      }
      $this->Database->update('NotifyUser', '`Id`='.$User['Id'], array('LastTime' => TimeToMysqlDateTime($Time)));

      if(($User['Category'] == 4) and ($Content !='')) {
        $Mail = new Mail();
        $Mail->Subject = 'Notification';
        $Mail->AddTo($User['Value'], $User['Name']);
        $Mail->AddBody(strip_tags($Content), 'text/plain');
        $Mail->AddBody($Content, 'text/html');
        $Mail->Send();
        $Output .= 'Sent email to '.$User['Value'].' ('.$User['Name'].')<br/>';
        $Output .= $Content;
      }

      return($Output);
    }
  }

  function DoInstall()
  {
    $this->Database->query('CREATE TABLE IF NOT EXISTS `NotifyCategory` (
      `Id` int(11) NOT NULL AUTO_INCREMENT,
      `Name` varchar(255) NOT NULL,
      `SysName` varchar(255) NOT NULL,
      PRIMARY KEY (`Id`)
    ) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARSET=utf8;');

    $this->Database->query("INSERT INTO `NotifyCategory` (`Id`, `Name`, `SysName`) VALUES
    (1, 'Dostupnost zařízení (ping)', 'NetworkReachability'),
    (2, 'Dostupnost URL', 'URL'),
    (3, 'Minimální úroveň signálu', 'WirelessSignal'),
    (4, 'Dostupnost síťového portu', 'NetworkPort'),
    (5, 'Minimální odezva', 'NetworkLatency'),
    (6, 'Minimální propustnost', 'NetworkBandwidth');");

    $this->Database->query('
  CREATE TABLE IF NOT EXISTS `NotifyUser` (
  `Id` int(11) NOT NULL AUTO_INCREMENT,
  `User` int(11) NOT NULL,
  `Contact` int(11) NOT NULL,
  `Period` int(11) NOT NULL,
  PRIMARY KEY (`Id`),
  KEY `User` (`User`),
  KEY `Contact` (`Contact`)
    ) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;');
    $this->Database->query('ALTER TABLE `NotifyUser`
ADD CONSTRAINT `NotifyUser_ibfk_1` FOREIGN KEY (`User`) REFERENCES `User` (`Id`),
ADD CONSTRAINT `NotifyUser_ibfk_2` FOREIGN KEY (`Contact`) REFERENCES `Contact` (`Id`);');

  }

  function DoUninstall()
  {
    $this->Database->query('DROP TABLE `NotifyUser`');
    $this->Database->query('DROP TABLE `NotifyCategory`');
  }
}

class ScheduleNotifyCheck extends SchedulerTask
{
  function Execute()
  {
    $Output = $this->System->ModuleManager->Modules['Notify']->Check();

    return($Output);
  }
}